/**
* DevExtreme (cjs/localization/persian.js)
* Version: 23.1.7
* Build date: Fri Apr 26 2024
*
* Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
"use strict";

exports.default = void 0;
var _core = _interopRequireDefault(require("./core"));
var _persianDate = _interopRequireDefault(require("persian-date"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
/* globals Intl */

var persianUtils = {
  isPersianLocale: function isPersianLocale() {
    if (!_core.default) {
      return false;
    }
    var locale = _core.default.locale();
    var intlLocale = new Intl.Locale(locale);
    if (intlLocale !== null && intlLocale !== void 0 && intlLocale.calendar) {
      return intlLocale.calendar === 'persian';
    }
    return intlLocale && intlLocale.baseName === 'fa-IR';
  },
  firstYearInDecade: function firstYearInDecade(date) {
    var pDate = new _persianDate.default(date);
    var pYear = pDate.year() - pDate.year() % 10;
    return pYear;
  },
  sameDecade: function sameDecade(date1, date2) {
    var date1PYear = new _persianDate.default(date1).year();
    var date2PYear = new _persianDate.default(date2).year();
    var startDate1 = date1PYear - date1PYear % 10;
    var startDate2 = date2PYear - date2PYear % 10;
    return date1 && date2 && startDate1 === startDate2;
  },
  sameCentury: function sameCentury(date1, date2) {
    var date1PYear = new _persianDate.default(date1).year();
    var date2PYear = new _persianDate.default(date2).year();
    var startCenturyDate1 = date1PYear - date1PYear % 100;
    var startCenturyDate2 = date2PYear - date2PYear % 100;
    return date1 && date2 && startCenturyDate1 === startCenturyDate2;
  },
  firstDecadeInCentury: function firstDecadeInCentury(date) {
    var pYear = new _persianDate.default(date).year();
    return date && pYear - pYear % 100;
  }
};
var _default = persianUtils;
exports.default = _default;
module.exports = exports.default;
module.exports.default = exports.default;
